package wrox.webservices.chapter8;

import java.net.*;
import java.text.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.datatype.binding.*; 
import com.ibm.uddi.datatype.business.*; 
import com.ibm.uddi.datatype.service.*;
import com.ibm.uddi.datatype.tmodel.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.util.*;

import org.apache.soap.*; 
import org.apache.soap.rpc.*;

/**
 * The FareClient class knows the tModel of the fare quoting 
 * service, and finds service based on that. It then invokes 
 * the getFare() method.
 *
 * This example is based on the premise that standards bodies
 * would define and publish tModels and individual  companies 
 * would provide implementations.
 */
public class FareClient {

  private static String TMODEL_NAME   = "fare_service_tmodel";

  public static void main (String args[]) {

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    UDDIProxy uddiProxy = new UDDIProxy ();

    /* Set the inquiry URL to the Test registry node at IBM */
    try {
      /* Had the business been published to the UDDI global registry
       * instead of the test registry, the inquiry URL could have been
       * set to any UDDI node (such as the one at Microsoft).
       */
      uddiProxy.setInquiryURL (Constants.IBM_INQUIRY_URL_TEST);
      System.out.println ("Inquiry URL : ["
                          + Constants.IBM_INQUIRY_URL_TEST + "]");
    } catch (MalformedURLException e) {
      System.err.println ("Malformed inquiry URL for the UDDI registry ["
                          + Constants.IBM_INQUIRY_URL_TEST + "]");
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }


    /* Assumption: There exists at least one tModel, Business, Service 
     *             that matches each of the searches. No error checking
     *             performed. We simply select the first found- in real
     *             life, there may be some business logic built in to
     *             select based on some other criteria.
     */

     /* Step 1: Find tModel by name and get its tModel key,
      *      OR have a well known tModel key
      *
      *      The OverviewURL of the tModel would point to the wsdl 
      *      description of the service.
      */
     
     /* get all tModelInfos matching the TMODEL_NAME */

     TModelList tModelList = null;
     try {
       tModelList = uddiProxy.find_tModel (TMODEL_NAME, null, 0);
      } catch (Exception e) { /* Either UDDI or SOAP Exception */
       System.err.println ("Error finding tModel : " 
                           + e.getMessage());
       e.printStackTrace ();
       System.exit (1); /* fatal error */
     }

     Vector tModelVector = tModelList.getTModelInfos().getTModelInfoVector();
     /* Assuming that there is one and only one tModel matching TMODEL_NAME */
     if (tModelVector.isEmpty()) {
       System.err.println ("No tModel matching name " + TMODEL_NAME 
                           + " found. Sorry!");
       System.exit (1);
     }
     String tModelKey = ((TModelInfo)tModelVector.elementAt(0)).getTModelKey ();

     /* Now that we have the required tModel key, drill down and get the 
       tModelDetail */

     TModelDetail tModelDetail = null;
     try {
         tModelDetail = uddiProxy.get_tModelDetail (tModelKey);
     } catch (Exception e) {
       System.err.println ("Exception finding tModel detail: " 
                           + e.getMessage ());
       e.printStackTrace ();
       System.exit (1); /* fatal error */
     }

     TModel tModel = (TModel)tModelDetail.getTModelVector().elementAt(0);
     String overviewURL = tModel.getOverviewDoc().getOverviewURLString();
     System.out.println ("OverviewURL of service is " + overviewURL);


     /* Step 2: Find businesses (businessEntity) that have a service that
      *      implements this tModel and and use some business logic to 
      *      select one from them.
      */
      Vector tmpTModelVector = new Vector();
      tmpTModelVector.addElement (tModelKey); /* found earlier */
      TModelBag tModelBag = new TModelBag (tmpTModelVector);
      BusinessList businessList = null;
      try {
        businessList = uddiProxy.find_business (tModelBag, null, 0);
      } catch (Exception e) {
        System.err.println ("Exception finding business entity: " 
                            + e.getMessage ());
        e.printStackTrace ();
        System.exit (1); /* fatal error */
      }

     Vector businessVector = businessList.getBusinessInfos().getBusinessInfoVector();
     /* Select one business out of the list- we select the first one */
     if (businessVector.isEmpty()) {
       System.err.println ("No Business found implemenenting tModel "
                           + TMODEL_NAME + ". Sorry!");
       System.exit (1);
     }
     String businessKey = ((BusinessInfo)businessVector.elementAt(0)).getBusinessKey ();

     /* Now that we have the required business key, we can drill down if
      * for additional information - the businessDetail structure. For
      * now, we dont need to do that.
      */

     /* Step 3: Locate the service (serviceInfo) within the business
      *      entity.
      *
      *      The AccessPoint would have the service URL - which in
      *      this case is the URL of the SOAP router/listener.
      */

      ServiceList serviceList = null;
      try {
        serviceList = uddiProxy.find_service (businessKey, tModelBag, null, 0);
      } catch (Exception e) {
        System.err.println ("Exception finding business service: " 
                            + e.getMessage ());
        e.printStackTrace ();
        System.exit (1); /* fatal error */
      }

     Vector serviceVector = serviceList.getServiceInfos().getServiceInfoVector();
     /* Taking the first business service matching this search */
     String serviceKey = ((ServiceInfo)serviceVector.elementAt(0)).getServiceKey();

     /* Now, drill down to get the ServiceDetail */

     ServiceDetail serviceDetail = null;
     try {
         serviceDetail = uddiProxy.get_serviceDetail (serviceKey);
     } catch (Exception e) {
       System.err.println ("Exception finding service detail: " 
                           + e.getMessage ());
       e.printStackTrace ();
       System.exit (1); /* fatal error */
     }

     BusinessService businessService = (BusinessService)serviceDetail.getBusinessServiceVector().elementAt(0);
     BindingTemplates bindingTemplates = businessService.getBindingTemplates();
     BindingTemplate bindingTemplate = (BindingTemplate) bindingTemplates.getBindingTemplateVector().elementAt(0);
     String accessPoint = bindingTemplate.getAccessPoint().getText();
     System.out.println ("accessPoint of service is " + accessPoint);

     /* Step 4: Bind and Invoke */

     /* The Apache SOAP v2.1 implementation does not make use of the
      * WSDL description of a service- most other implementation do.
      * Since we are using Apache SOAP here, the overviewURL is not
      * used for now.
      */

      /* Build the SOAP request call - fare request for a flight from
       * San Jose, California to Teipei, Taiwan on April 1, 2002.
       */
      Call call = new Call ();
      String serviceURN = "urn:fareservice-service";
      call.setTargetObjectURI (serviceURN);
      call.setMethodName ("getFare");
      call.setEncodingStyleURI (org.apache.soap.Constants.NS_URI_SOAP_ENC); 
      Vector params = new Vector(); 
      params.addElement (new Parameter ("source", String.class, "SJC", null)); 
      params.addElement (new Parameter ("destination", String.class, "TPE", null)); 
      SimpleDateFormat dateFormat = new SimpleDateFormat ("yyyy.MM.dd");
      Date date = null;
      try {
        date = dateFormat.parse ("2002.04.01");
      } catch (Exception e) {/* ignore */}
      params.addElement (new Parameter ("date", Date.class, date, null)); 
      call.setParams (params); 

      /* Invoke the method */
      URL serviceURL = null;
      try {
        serviceURL = new URL (accessPoint);
      } catch (Exception e) {
       System.err.println ("AccessPoint URL [" + accessPoint + "] is malformed: "
                           + e.getMessage ());
       System.exit (1); /* fatal error */
      }

      Response response = null;
      try {
        System.out.println ("Posting getFare request to " + accessPoint);
	response = call.invoke (serviceURL, null);
      }  catch (SOAPException e) {
        System.err.println ("Exception invoking method: "
	                   + e.getFaultCode() + ": "
                           + e.getMessage ());
        e.printStackTrace ();
        System.exit (1); /* fatal error */
      }

      if (response.generatedFault()) { /* SOAP Fault code returned */
        Fault fault = response.getFault();
        System.err.println ("Error invoking method: [" + fault.getFaultCode()
	                    + "] " + fault.getFaultString());
      } else { /* Success! */
        Parameter result = response.getReturnValue ();
	System.out.println ("Requested fare is $" + result.getValue());
      }
  }
}
