package wrox.webservices.chapter8;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.datatype.tmodel.*;
import com.ibm.uddi.datatype.business.*; 
import com.ibm.uddi.util.*;

/**
 * The SaveBusiness class publishes a Business Listing for a 
 * fictitious Airline company in the UDDI registry.
 */
public class SaveBusiness {

  /* The username and password to interact with the UDDI registry */
  private static String USERNAME = "yourname";
  private static String PASSWORD = "yourpassword";

  /* The business information */
  private static String BUSINESS_NAME        = "Crashes R'Us Airlines";
  private static String BUSINESS_DESCRIPTION =
  "Crashes R'Us Airlines aims to provide a memorable flight experience- "
  + "remember to bring a bathing suit on our trans-Pacific flights!";
  private static String CONTACT_PERSON       = "John J. Doe";
  private static String CONTACT_EMAIL        = "john.j.doe@crashesrus.com";
  private static String CONTACT_ADDRESS_LN1  = "123 Airline Ave";
  private static String CONTACT_ADDRESS_LN2  = "San Jose";
  private static String CONTACT_ADDRESS_LN3  = "CA 95192";
  private static String CONTACT_PHONE        = "1-408-555-1234";

  /* Helper method for constructing a vector with one element in it */
  private static Vector constructVector (Object object) {
    Vector vector = new Vector ();
    vector.addElement (object);
    return vector;
  }

  public static void main (String args[]) {

    /* Settings for HTTPS, SSL support */
    System.setProperty ("java.protocol.handler.pkgs", 
                        "com.ibm.net.ssl.internal.www.protocol");
    java.security.Security.addProvider (new com.ibm.jsse.JSSEProvider ());

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    UDDIProxy uddiProxy = new UDDIProxy ();

    /* Set the publish URL to the Test registry node at IBM */
    try {
      uddiProxy.setPublishURL (Constants.IBM_PUBLISH_URL_TEST);
      System.out.println ("Publishing businessEntity to " 
                          + Constants.IBM_PUBLISH_URL_TEST);
    } catch (MalformedURLException e) {
      System.err.println ("Malformed publish URL for the UDDI registry ["
                          + Constants.IBM_PUBLISH_URL_TEST + "]");
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* Get an authorization token */
    AuthToken authToken = null;
    try {
      authToken = uddiProxy.get_authToken(USERNAME, PASSWORD);
    } catch (Exception e) { /* Either UDDI or SOAP Exception */
      System.err.println ("Error getting authorization token: " 
                          + e.getMessage());
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* Create and populate our business entity object.
     * The business key is passed as an empty string- this causes it to
     * be assigned by the operator node
     */
    BusinessEntity businessEntity = new BusinessEntity ("", BUSINESS_NAME);
   
    /* Set Business Description */
    businessEntity.setDefaultDescriptionString (BUSINESS_DESCRIPTION);
    
    /* Set Contact Info */
    Contact  contact  = new Contact ();
    contact.setPersonName (CONTACT_PERSON);

    Address address = new Address ();
    Vector addressStrings = new Vector ();
    addressStrings.addElement (CONTACT_ADDRESS_LN1);
    addressStrings.addElement (CONTACT_ADDRESS_LN2);
    addressStrings.addElement (CONTACT_ADDRESS_LN3);
    address.setAddressLineStrings (addressStrings);
    contact.setAddressVector (constructVector (address));

    Email email = new Email (CONTACT_EMAIL);
    contact.setEmailVector (constructVector (email));

    Phone phone = new Phone (CONTACT_PHONE);
    contact.setPhoneVector (constructVector (phone));

    Contacts contacts = new Contacts ();
    contacts.setContactVector (constructVector (contact));
    businessEntity.setContacts (contacts);

    /* Set business classification -
     *  We use NAICS (North American Industry Classification System) codes
     *  to classify this business. The particular code that we use is:
     *    481111  Scheduled Passenger Air Transportation
     *  See www.naics.com for additional details.
     */
    KeyedReference keyedReference = new KeyedReference ();
    keyedReference.setKeyName ("NAICS: Scheduled Passenger Air Transportation");
    keyedReference.setKeyValue ("481111");
    keyedReference.setTModelKey (TModel.NAICS_TMODEL_KEY);

    CategoryBag categoryBag = new CategoryBag ();
    categoryBag.setKeyedReferenceVector (constructVector (keyedReference));
    businessEntity.setCategoryBag (categoryBag);

    /* Create a Vector of business entity objects, and add 
     * the BusinessEntity created to it.
     */
    Vector businessEntities = new Vector ();
    businessEntities.addElement (businessEntity);
   
    /* Finally, update this business entity information in the registry */
    System.out.println ("Attempting to publish businessEntity for " +
                        BUSINESS_NAME);
    BusinessDetail businessDetail = null;
    try {
      businessDetail = uddiProxy.save_business (authToken.getAuthInfoString(), 
                                                businessEntities);
    } catch (Exception e) { /* Either UDDI or SOAP Exception */
      System.err.println ("Error saving business entity: " 
                          + e.getMessage());
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* Get the business key assigned for our company */
    Vector businessEntityVector = businessDetail.getBusinessEntityVector();

    /* Since we saved only one business entity, the resulting business
     * detail should have only one business entity in it 
     */
    BusinessEntity thisBusinessEntity = (BusinessEntity)
                                          (businessEntityVector.elementAt(0));
    String businessKey = thisBusinessEntity.getBusinessKey ();

    System.out.println ("The " + BUSINESS_NAME 
                   + " Business Entity has been successfully updated "
                   + " in the UDDI registry and assigned the Business Key: "
                   + businessKey);
  }
}
