package wrox.webservices.UDDIImpl;

import java.net.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.datatype.binding.*;
import com.ibm.uddi.datatype.business.*; 
import com.ibm.uddi.datatype.tmodel.*;
import com.ibm.uddi.datatype.service.*;
import com.ibm.uddi.util.*;

import org.apache.soap.SOAPException;
import org.apache.soap.transport.http.SOAPHTTPConnection;

/**
 * The SaveBusiness class publishes a Business Listing for a 
 * fictitious Airline company in the UDDI registry.
 */
public class SaveBusiness {

  /* The username and password to interact with the UDDI registry */
  private final static String USERNAME = "phillipdj";
  private final static String PASSWORD = "wroxpress";

  /* The business information */
  private final static String BUSINESS_NAME        = "Crashes R'Us Airlines";
  private final static String BUSINESS_DESCRIPTION =
  "Crashes R'Us Airlines aims to provide a memorable flight experience- "
  + "remember to bring a bathing suit on our trans-Pacific flights!";
  private final static String CONTACT_PERSON       = "John J. Doe";
  private final static String CONTACT_EMAIL        = "john.j.doe@crashesrus.com";
  private final static String CONTACT_ADDRESS_LN1  = "123 Airline Ave";
  private final static String CONTACT_ADDRESS_LN2  = "San Jose";
  private final static String CONTACT_ADDRESS_LN3  = "CA 95192";
  private final static String CONTACT_PHONE        = "1-408-555-1234";
  private final static String SERVICE_NAME         = "fare_service";
  private final static String SERVICE_URL          = "http://127.0.0.1:8080/soap/servlet/rpcrouter";
  private final static String SERVICE_DESC         = "SOAP service for fare quotes";


  /* Helper method for constructing a vector with one element in it */
  private static Vector constructVector (Object object) {
    Vector vector = new Vector ();
    vector.addElement (object);
    return vector;
  }

  public static void main (String args[]) {
    
    /* Process command line arguments */
    if (args.length != 1) {
      System.out.println 
         ("usage: java wrox.webserices.UDDIImpl.SaveBusiness UUID:94B5B570-80B5-11D5-A76E-0004AC49CC1E");
      System.exit (1);
    }

    /* Settings for HTTPS, SSL support */
    System.setProperty ("java.protocol.handler.pkgs", 
                        "com.ibm.net.ssl.internal.www.protocol");
    java.security.Security.addProvider (new com.ibm.jsse.JSSEProvider ());

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
    UDDIProxy uddiProxy = new UDDIProxy ();
    /* Setting for the proxy server- in case you are behind one */
    
     SOAPHTTPConnection soapHTTPConnection = new SOAPHTTPConnection ();
     soapHTTPConnection.setProxyHost ("192.168.0.46");
     soapHTTPConnection.setProxyPort (80);
     uddiProxy.setTransport (soapHTTPConnection);
    */


    /* Set the publish URL to the Test registry node at IBM */
    try {
      uddiProxy.setPublishURL (UDDIOperatorURLs.IBM_PUBLISH_URL_TEST);
      System.out.println ("Publishing businessEntity to " 
                          + UDDIOperatorURLs.IBM_PUBLISH_URL_TEST);
    } catch (MalformedURLException e) {
      System.err.println ("Malformed publish URL for the UDDI registry ["
                          + UDDIOperatorURLs.IBM_PUBLISH_URL_TEST + "]");
      e.printStackTrace ();
      System.exit (1); /* fatal error */
    }

    /* Get an authentication token */
    AuthToken authToken = null;
    try {
      authToken = uddiProxy.get_authToken(USERNAME, PASSWORD);
    } catch (UDDIException e) {
      System.err.println ("UDDI error getting authentication token: [" 
                          + e.getFaultCode() + "]" + e.getFaultString()
			 );
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("More error information : " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP error getting authentication token: " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */

    }

    /* Create and populate our business entity object.
     * The business key is passed as an empty string- this causes it to
     * be assigned by the operator node
     */
    BusinessEntity businessEntity = new BusinessEntity ("", BUSINESS_NAME);
   
    /* Set Business Description */
    businessEntity.setDefaultDescriptionString (BUSINESS_DESCRIPTION);
    
    /* Set Contact Info */
    Contact  contact  = new Contact ();
    contact.setPersonName (CONTACT_PERSON);

    Address address = new Address ();
    Vector addressStrings = new Vector ();
    addressStrings.addElement (CONTACT_ADDRESS_LN1);
    addressStrings.addElement (CONTACT_ADDRESS_LN2);
    addressStrings.addElement (CONTACT_ADDRESS_LN3);
    address.setAddressLineStrings (addressStrings);
    contact.setAddressVector (constructVector (address));

    Email email = new Email (CONTACT_EMAIL);
    contact.setEmailVector (constructVector (email));

    Phone phone = new Phone (CONTACT_PHONE);
    contact.setPhoneVector (constructVector (phone));

    Contacts contacts = new Contacts ();
    contacts.setContactVector (constructVector (contact));
    businessEntity.setContacts (contacts);

    /* Set business classification -
     *  We use NAICS (North American Industry Classification System) codes
     *  to classify this business. The particular code that we use is:
     *    481111  Scheduled Passenger Air Transportation
     *  See www.naics.com for additional details.
     */
    KeyedReference keyedReference = new KeyedReference ();
    keyedReference.setKeyName ("NAICS: Scheduled Passenger Air Transportation");
    keyedReference.setKeyValue ("481111");
    keyedReference.setTModelKey (TModel.NAICS_TMODEL_KEY);

    CategoryBag categoryBag = new CategoryBag ();
    categoryBag.setKeyedReferenceVector (constructVector (keyedReference));
    businessEntity.setCategoryBag (categoryBag);

    /* Construct the bindingTemplate structure */
    String tModelKey = args[0];
    TModelInstanceInfo tModelInstanceInfo = new TModelInstanceInfo (tModelKey);
    TModelInstanceDetails tModelInstanceDetails = new TModelInstanceDetails();
    tModelInstanceDetails.setTModelInstanceInfoVector (constructVector (tModelInstanceInfo));
    BindingTemplate bindingTemplate = new BindingTemplate ();
    bindingTemplate.setTModelInstanceDetails (tModelInstanceDetails);
    bindingTemplate.setAccessPoint (new AccessPoint (SERVICE_URL, "http"));
    bindingTemplate.setDefaultDescriptionString (SERVICE_DESC);

    /* Construct the businessService structure */
    BindingTemplates bindingTemplates = new BindingTemplates ();
    bindingTemplates.setBindingTemplateVector (constructVector(bindingTemplate));
    BusinessService businessService = new BusinessService ("",
                                       SERVICE_NAME, bindingTemplates);

    /* Add to businessEntity */
    BusinessServices businessServices = new BusinessServices ();
    businessServices.setBusinessServiceVector (constructVector (businessService));
    businessEntity.setBusinessServices (businessServices);

    /* Create a Vector of business entity objects, and add 
     * the BusinessEntity created to it.
     */
    Vector businessEntities = new Vector ();
    businessEntities.addElement (businessEntity);
   
    /* Finally, update this business entity information in the registry */
    System.out.println ("Attempting to publish businessEntity for " +
                        BUSINESS_NAME);
    BusinessDetail businessDetail = null;
    try {
      businessDetail = uddiProxy.save_business (authToken.getAuthInfoString(), 
                                                businessEntities);
    } catch (UDDIException e) {
      System.err.println ("UDDI related error while saving business entity: [" 
                          + e.getFaultCode() + "]" + e.getFaultString()
			 );
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("More error information : " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP related error saving business entity: " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */

    }

    
    /* Finally, invalidate the authentication token */
    try {
      uddiProxy.discard_authToken(authToken.getAuthInfo());
    } catch (Exception e) { /* ignore errors */ }

    /* Get the business key assigned for our company */
    Vector businessEntityVector = businessDetail.getBusinessEntityVector();

    /* Since we saved only one business entity, the resulting business
     * detail should have only one business entity in it 
     */
    BusinessEntity thisBusinessEntity = (BusinessEntity)
                                          (businessEntityVector.elementAt(0));
    String businessKey = thisBusinessEntity.getBusinessKey ();

    System.out.println ("The " + BUSINESS_NAME 
                   + " Business Entity has been successfully updated "
                   + " in the UDDI registry and assigned the Business Key: "
                   + businessKey);

  }
}
