package wrox.webservices.UDDIImpl;

import java.net.*;
import java.text.*;
import java.util.*;
import java.io.*;

import com.ibm.uddi.*;
import com.ibm.uddi.client.*;
import com.ibm.uddi.datatype.binding.*; 
import com.ibm.uddi.datatype.business.*; 
import com.ibm.uddi.datatype.service.*;
import com.ibm.uddi.datatype.tmodel.*;
import com.ibm.uddi.response.*;
import com.ibm.uddi.util.*;

import org.apache.soap.SOAPException;
import org.apache.soap.transport.http.SOAPHTTPConnection;

/**
 * UDDIClient class uses the UDDI4J Inquiry API to navigate
 * the UDDI registry. This example shows how a vendor company
 * tries to get information about buying widgets from a 
 * vendor company- "Foobar Widgets Inc".
 */
public class UDDIClient {

  private final static String BUSINESS_NAME = "Foobar Widgets Inc";
  private final static String SERVICE_NAME  = "Buy Widgets";

  public static void main (String args[]) {

    /* Construct a UDDIProxy object- this represents a UDDI server 
     * and the actions that can be invoked against it.
     */
        UDDIProxy uddiProxy = new UDDIProxy ();

    /* Setting for the proxy server- in case you are behind one */
    /*
     SOAPHTTPConnection soapHTTPConnection = new SOAPHTTPConnection ();
     soapHTTPConnection.setUserName ("proxy-username");
     soapHTTPConnection.setPassword ("proxy-password");
     soapHTTPConnection.setProxyHost ("proxyhostname");
     soapHTTPConnection.setProxyPort (8080);
     uddiProxy.setTransport (soapHTTPConnection);
    */

    /* Select which UDDI node to send the query to */
    if (args.length < 1) {
       System.out.println ("usage: UDDIClient <IBM_PROD|MS_PROD>");
       System.exit (1);
    }

    String inquiryUrl = null;
    if (args[0].equals ("IBM_PROD")) {
      inquiryUrl = UDDIOperatorURLs.IBM_INQUIRY_URL_PRODUCTION;
      System.out.println ("Inquiry URL set to IBM UDDI node [" 
                          + inquiryUrl + "]");
    } else { /* MS_PROD */
      inquiryUrl = UDDIOperatorURLs.MICROSOFT_INQUIRY_URL_PRODUCTION;
      System.out.println ("Inquiry URL set to Microsoft UDDI node [" 
                          + inquiryUrl + "]");
    }

    try {
      uddiProxy.setInquiryURL (inquiryUrl);
    } catch (MalformedURLException e) {
      System.err.println ("Malformed inquiry URL [" + inquiryUrl + "]");
      System.exit (1); /* fatal error */
    }

    /* Find all businesses matching the partial name "Foobar Widgets" */
    BusinessList businessList = null;
    try {
        businessList = uddiProxy.find_business (BUSINESS_NAME, null, 0);
    } catch (UDDIException e) {
      System.err.println ("UDDI error finding business: [" 
                          + e.getFaultCode() + "]" + e.getFaultString()
			 );
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("More error information : " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP error finding business " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */

    }

    Vector businessInfoVector = 
       businessList.getBusinessInfos().getBusinessInfoVector();

    if (businessInfoVector.size() == 0) {
      System.err.println ("No businesses found matching " +
                          BUSINESS_NAME + ". Quitting.");
      System.exit (1);
    }
    System.out.println ("Found " + businessInfoVector.size()
                        + " businesses matching the name" + BUSINESS_NAME 
			+ ". These are: \n");


    for (int i = 0; i < businessInfoVector.size(); i ++) {
      System.out.println (
          "[" + i + "] "
          + ((BusinessInfo)businessInfoVector.elementAt(i)).getNameString()
	  + " [businessKey=" + 
          ((BusinessInfo)businessInfoVector.elementAt(i)).getBusinessKey()
	  + "]");
    }

    /* Taking the first found business */
    String businessKey = ((BusinessInfo)businessInfoVector.elementAt(0)).getBusinessKey();

    System.out.println ("Selected: " + BUSINESS_NAME + " with business key " +
                        businessKey);

    /* Drill down for more information about the business */
    Vector businessKeyVector = new Vector ();
    businessKeyVector.addElement (businessKey);

    BusinessDetail businessDetail = null;
    try {
        businessDetail = uddiProxy.get_businessDetail (businessKeyVector);
    } catch (UDDIException e) {
      System.err.println ("UDDI error finding business detail: [" 
                          + e.getFaultCode() + "]" + e.getFaultString()
			 );
      DispositionReport dispositionReport = e.getDispositionReport ();
      System.err.println ("More error information : " 
                          + dispositionReport.getErrCode() + ": "
			  + dispositionReport.getErrInfoText());
      System.exit (1); /* fatal error */
    } catch (SOAPException e) {
      System.err.println ("SOAP error finding business detail: " 
                          + e.getFaultCode() + ": " + e.getMessage());
      System.exit (1); /* fatal error */

    }

    BusinessEntity businessEntity = (BusinessEntity) businessDetail.getBusinessEntityVector().elementAt(0);
    Vector businessServiceVector = businessEntity.getBusinessServices().getBusinessServiceVector();

    System.out.println ("Found " + businessServiceVector.size()
                   + " services in this business entity. These are: \n");

    BusinessService businessService = null;
    for (int i = 0; i < businessServiceVector.size(); i++) {
      String serviceName = ((BusinessService)businessServiceVector.elementAt(i)).getNameString();
      System.out.println ("[" + i + "] " + serviceName);
      if (serviceName.equals (SERVICE_NAME)) {
	businessService = (BusinessService)businessServiceVector.elementAt(i);
      }
    }

    if (businessService == null) {
      System.err.println ("No service found matching " +
                          SERVICE_NAME + ". Quitting.");
      System.exit (1);
    }

    System.out.println ("Selected " + SERVICE_NAME + 
                        " service with service key " +
                        businessService.getServiceKey());

    /* Get the binding templates- information on how to access the service */
    Vector bindingTemplateVector = 
       businessService.getBindingTemplates().getBindingTemplateVector();
    /* Find the access point for the first binding template */
    AccessPoint accessPoint = ((BindingTemplate)bindingTemplateVector.elementAt(0)).getAccessPoint();

    System.out.println ("The " + SERVICE_NAME 
                        + " service can be accessed via "
                        + accessPoint.getURLType() 
			+ ": "
			+ accessPoint.getText());
  }
}
