/*
 * FileSystem.java
 */

package com.markrichman.filesystem;

import java.io.*;
import java.sql.*;
import java.util.Date;
import java.util.Enumeration;
import javax.activation.*;
import javax.mail.internet.*;
import javax.servlet.* ;
import javax.servlet.http.* ;
import org.apache.soap.encoding.soapenc.Base64;
import org.apache.soap.rpc.SOAPContext;
import org.apache.soap.util.mime.*;
import org.apache.soap.util.xml.*;

/**
 * @author Mark A. Richman
 * @version 1.0
 */
public class Filesystem {

    /**
     * Copies a file to a new location.
     */
    public void copyTo(SOAPContext ctx, String sourcePath, String destPath) throws Exception {
        File fnew = new File(getHomedir(getUsername(ctx)) + "/" + destPath);
        File fold = new File(getHomedir(getUsername(ctx)) + "/" + sourcePath);

        InputStream in = null;
        OutputStream out = null;
        try {
            in = new FileInputStream(fold);
            out = new FileOutputStream(fnew);
            while (true) {
                int data = in.read();
                if (data == -1) {
                    break;
                }
                out.write(data);
            }
            in.close();
            out.close();
            } 
        finally {
            if (in != null) {
                in.close();
                }
            if (out != null) {
                out.close();
                }
        }
    }

    /**
     * Creates a new file.
     */
    public void createNewFile(SOAPContext ctx, String filePath) throws Exception {

        File fnew = new File(getHomedir(getUsername(ctx)) + "/" + filePath);

        try {
            if(!fnew.createNewFile()) {
                throw new Exception("File already exists: " + filePath);
            }
        }
        catch(Exception e) {
            throw new Exception(e.getMessage());
        }

        // Write the data
        MimeBodyPart mbp;
        DataHandler dh;
        Object o;
        InputStream is;
        FileOutputStream fos = new FileOutputStream(fnew);

        try {
            mbp = ctx.getBodyPart(1);
            dh = mbp.getDataHandler();
            is = dh.getInputStream();

            int c;
            while ((c = is.read()) != -1)
                fos.write(c);
        } 
        catch(Exception e) {
            throw new Exception(e.getMessage());
        }

        return;
    }

    /**
     * Deletes a file.
     */
    public boolean delete(SOAPContext ctx, String filePath) throws Exception {
        File fnew = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return fnew.delete();
    }

    /**
     * Tests whether the file denoted by the filePath exists.
     */
    public boolean exists(SOAPContext ctx, String filePath) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.exists();
    }

    /**
     * Retrieves a file.
     */
    /*public void getFile(SOAPContext ctx, String filePath) throws Exception {

        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        if(!f.exists())
            throw new Exception("File not found.");

        try {
            MimeBodyPart mbp;
            DataHandler dh;
            Object o;
            InputStream is;
            FileOutputStream fos = new FileOutputStream(f);

            mbp = (MimeBodyPart)ctx.getBodyPart(1);
            dh = mbp.getDataHandler();
            is = dh.getInputStream();

            int c;
            while ((c = is.read()) != -1)
                fos.write(c);
        } 
        catch(Exception e) {
            System.err.println(e.getMessage());
        }
    }*/
    
    public DataHandler getFile(SOAPContext ctx, String filePath) throws Exception {
	    
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        if(!f.exists()) 
       	     throw new Exception("File not found.");

	    try {
        	System.out.println(f.getAbsolutePath());
	        DataSource ds = new FileDataSource(f);
	        System.out.println("DS Name: "+ds.getName());
	        System.out.println("DS Type: "+ds.getContentType());
            DataHandler dh = new DataHandler(ds);
            System.out.println("DH Name: "+dh.getName());
			return dh;
	    } 
	    catch(Exception e) {
	        System.err.println(e.getMessage());
            throw new Exception(e.getMessage()); 
        }
	}


    /**
     * Returns the length of the file denoted by the filePath.
     */
    public long length(SOAPContext ctx, String filePath) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.length();  
    }

    /**
     * Returns the pathname string of the filePath's parent, 
    * or null if this pathname does not name a parent directory. 
     */
    public String getParent(SOAPContext ctx, String filePath) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.getParent();   
    }

    /**
     * Tests whether the file denoted by the filePath is a directory.
     */
    public boolean isDirectory(SOAPContext ctx, String filePath) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.isDirectory();
    }

    /**
     * Tests whether the file denoted by the filePath is a normal file. 
    * A file is normal if it is not a directory and, in addition, satisfies other 
    * system-dependent criteria. Any non-directory file created by a Java 
    * application is guaranteed to be a normal file.
     */
    public boolean isFile(SOAPContext ctx, String filePath) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.isFile();
    }

    /**
     * Returns the time that the file denoted by the filePath was last modified.
     */
    public long lastModified(SOAPContext ctx, String filePath) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.lastModified();    
    }

    /**
     * Returns an array of strings naming the files and directories in the directory 
    * denoted by the filePath. If this abstract pathname does not denote a directory, 
    * then this method returns null. Otherwise an array of strings is returned, one 
    * for each file or directory in the directory. Names denoting the directory 
    * itself and the directory's parent directory are not included in the result. 
    * Each string is a file name rather than a complete path. There is no guarantee 
    * that the name strings in the resulting array will appear in any specific order; 
    * they are not, in particular, guaranteed to appear in alphabetical order.
     */
    public String[] list(SOAPContext ctx, String filePath) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.list();    
    }

    /**
     * Creates the directory named by this filePath.
     */
    public boolean mkdir(SOAPContext ctx, String filePath) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.mkdir();   
    }

    /**
     * Creates the directory named by the filePath, including any 
    * necessary but nonexistent parent directories. Note that if this operation 
    * fails it may have succeeded in creating some of the necessary parent directories.
     */
    public boolean mkdirs(SOAPContext ctx, String filePath) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.mkdirs();; 
    }

    /**
    * Renames the file denoted by the filePath
    * @param newName The new name of the file.
    */
    public boolean renameTo(SOAPContext ctx, String filePath, String newName) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.renameTo(new File(getUsername(ctx) + "/" + newName));
    }

    /**
    * Sets the last-modified time of the file or directory named by the filePath.
    * @param lastModified The new last-modified time, measured in milliseconds 
    * since the epoch (00:00:00 GMT, January 1, 1970)
    */
    public boolean setLastModified(SOAPContext ctx, String filePath, long time) throws Exception {
        File f = new File(getHomedir(getUsername(ctx)) + "/" + filePath);
        return f.setLastModified(time);
    }

    /**
     * Gets the username from the HTTP Basic Authentication info,
    * specifically, the "Authorization" header.
     */
    protected String getUsername(SOAPContext ctx) {
        HttpServletRequest req = (HttpServletRequest)ctx.getProperty("HttpServletRequest");
        String authorization = req.getHeader("Authorization");
        authorization = authorization.substring(authorization.indexOf(" ")); // strip "Basic "
        byte[] bytes = Base64.decode(authorization);
        String decoded = new String(bytes);
        int i = decoded.indexOf(":");
        String username = decoded.substring(0,i);
        return username;    
    }

    /**
    * Gets the user's home directory via the FilesystemDAO
    * data access object.
    */
    protected String getHomedir(String username) throws Exception {
        FilesystemDAO dao = new FilesystemDAO();
        try {
            return dao.getHomedir(username);
        }
        catch (Exception e) {
            throw new Exception(e.getMessage());
        }

    }

}
