/**
 * FilesystemProvider.java
 */

package com.markrichman.filesystem;

import java.io.* ;
import java.util.* ;
import javax.servlet.* ;
import javax.servlet.http.* ;
import org.apache.soap.* ;
import org.apache.soap.encoding.soapenc.Base64;
import org.apache.soap.rpc.* ;
import org.apache.soap.server.* ;
import org.apache.soap.server.http.* ;
import org.apache.soap.util.* ;

/**
 * The <code>FilesystemProvider</code> class is a Pluggable Provider
 * for Apache SOAP which implements authentication and 
 * authorization for web services using HTTP Basic 
 * Authentication. This class is invoked by the Apache SOAP
 * framework, and is constructorless.
 *
 * @author Mark A. Richman
 * @version 1.0
 */

public class FilesystemProvider implements Provider
{
	protected DeploymentDescriptor dd;
	protected Envelope             envelope;
	protected Call                 call;
	protected String               methodName;
	protected String               targetObjectURI;
	protected HttpServlet          servlet;
	protected HttpSession          session;
	protected HttpServletRequest   req;
	protected Object			   targetObject;
	protected FilesystemDAO		   dao;
	protected Hashtable			   props;
	protected Enumeration		   keys;
	protected String 			   authorization;
	protected String			   username;
	protected String			   password;

	/**
	 * Locates the web service.
	 */
	public void locate( DeploymentDescriptor dd,
		Envelope env,
		Call call,
		String methodName,
		String targetObjectURI,
		SOAPContext reqContext)
	throws SOAPException { 

		HttpServlet servlet = (HttpServlet)reqContext.getProperty( Constants.BAG_HTTPSERVLET );
		HttpSession session = (HttpSession)reqContext.getProperty( Constants.BAG_HTTPSESSION );
		HttpServletRequest req = (HttpServletRequest)reqContext.getProperty(Constants.BAG_HTTPSERVLETREQUEST);

		System.err.println( "=============================================" );
		System.err.println( "In AuthProvider.locate()" );
		System.err.println( "Method: " + methodName );
		System.err.println( "URI: " + targetObjectURI );
		System.err.println( "DD.ServiceClass: " + dd.getServiceClass() );
		System.err.println( "DD.ProviderClass: " + dd.getProviderClass() );
		System.err.println( "Call.MethodName: " + call.getMethodName() );

		props = dd.getProps();

		if (props != null) keys  = props.keys();

		System.err.println( "Options:" );

		if (props != null && keys != null) {
			while ( keys.hasMoreElements() ) {
				String  key   = (String) keys.nextElement();
				String  value = (String) props.get( key );
				System.err.println( "Key: " + key + "  Value: " + value );
			}
		}

		this.dd              = dd ;
		this.envelope        = env ;
		this.call            = call ;
		this.methodName      = methodName ;
		this.targetObjectURI = targetObjectURI ;
		this.servlet         = servlet ;
		this.session         = session ;

		// Connect to datasource

		this.dao			 = new FilesystemDAO();
		try {
			dao.connect();
		}
		catch (Exception e) {
			throw new SOAPException( Constants.FAULT_CODE_SERVER, e.getMessage());
		}

		// decode HTTP Basic Authentication info
		System.err.println("Authenticating...");
		authorization = req.getHeader("authorization");
		authorization = authorization.substring(authorization.indexOf(" ")); // strip "Basic "
		byte[] bytes = Base64.decode(authorization);
		String decoded = new String(bytes);
		int i = decoded.indexOf(":");
		this.username = decoded.substring(0,i);
		this.password = decoded.substring(i+1,decoded.length());
		System.out.println("username: " + username);
		System.out.println("password: " + password);		

		if(!dao.isAuthenticated(username,password)) {
			System.err.println("Bad password");
			throw new SOAPException( Constants.FAULT_CODE_SERVER, "Bad password" );
		}

		// Authorize User
		System.err.println("Authorizing " + username + " for service " + targetObjectURI);
		if(!dao.isAuthorized(username,targetObjectURI)) {
			System.err.println("User not authorized");
			throw new SOAPException( Constants.FAULT_CODE_SERVER, "User not authorized" );
		}

		// Disconnect
		try  {
			dao.disconnect();
		}
		catch (Exception e) {
			throw new SOAPException( Constants.FAULT_CODE_SERVER, e.getMessage());
		}

		// Logic to locate/load the service:

		ServletConfig  config  = servlet.getServletConfig();
		ServletContext context = config.getServletContext();
		ServiceManager serviceManager =
			ServerHTTPUtils.getServiceManagerFromContext (context);

		// Default processing for 'java' and 'script' providers
		// call on a valid method name?
		if (!RPCRouter.validCall (dd, call)) {
			throw new SOAPException (Constants.FAULT_CODE_SERVER,
				"Method '" + call.getMethodName () +
				"' is not supported.");
		}

		System.err.println("Getting Target Object " + targetObjectURI);

		try {

			// get at the target object
			targetObject = ServerHTTPUtils.getTargetObject (serviceManager,
				dd, targetObjectURI,
				servlet, session,
                reqContext,
				context);

		} catch(Exception e) {
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
	};

	/**
	 * Invokes the web service.
	 */
	public void invoke(SOAPContext reqContext, SOAPContext resContext)
	throws SOAPException {
		System.err.println( "=============================================" );
		System.err.println( "In AuthProvider.invoke()" );

		// Add logic to invoke the service and get back the result here

		// If you want the client to use a new targetObjectURI for subsequence
		// calls place it here (targetObjectURI on 'new Response').
		//   i.e. we added an 'instance-id' to the urn
		try {
			Response resp = RPCRouter.invoke( dd, call, targetObject, reqContext, resContext );
			Envelope env = resp.buildEnvelope();
			StringWriter  sw = new StringWriter();
			env.marshall( sw, call.getSOAPMappingRegistry(), resContext );
			resContext.setRootPart( sw.toString(), Constants.HEADERVAL_CONTENT_TYPE_UTF8);
		}
		catch( Exception e ) {
			if ( e instanceof SOAPException ) {
				System.err.println( "=============================================" );
				System.err.println("SOAPException: " + e.getMessage());
				throw (SOAPException ) e ;
			}
			throw new SOAPException( Constants.FAULT_CODE_SERVER, e.toString() );
		}
	};
};

